#------------------------------------------------------------------------------
# Copyright 2008-2012 Istituto Nazionale di Fisica Nucleare (INFN)
# 
# Licensed under the EUPL, Version 1.1 only (the "Licence").
# You may not use this work except in compliance with the Licence. 
# You may obtain a copy of the Licence at:
# 
# http://joinup.ec.europa.eu/system/files/EN/EUPL%20v.1.1%20-%20Licence.pdf
# 
# Unless required by applicable law or agreed to in 
# writing, software distributed under the Licence is 
# distributed on an "AS IS" basis, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. 
# See the Licence for the specific language governing 
# permissions and limitations under the Licence.
#------------------------------------------------------------------------------

import os
import commands
import exceptions

import pycurl
import simplejson
import cStringIO

import datetime

from wnodes.cli.utils import utilities

class AuthOpenStackError(exceptions.Exception):
    pass

class TokenExpired(exceptions.Exception):
    pass

class Token(object):
    def __init__(self, path, endpoint):
        self.filename = ("%s/%s-%s.json"
            % (path, 'OpenStack', endpoint))

    def store_token(self, token):
        out_file = open(self.filename, "w")
        out_file.write(simplejson.dumps(token))
        out_file.close()

    def __is_expired(self, ot):
        ct = datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%SZ")
        
        if datetime.datetime.strptime(ct, "%Y-%m-%dT%H:%M:%SZ") > datetime.datetime.strptime(ot, "%Y-%m-%dT%H:%M:%SZ"):
            return True
        else:
            return False

    def get_id(self):
        if self.is_stored():
            tp_json = utilities.get_json_data(self.filename)

            if self.__is_expired(tp_json['expires']):
                msg = 'Redo Token'
                raise TokenExpired(msg)
        
            return tp_json['id']
        else:
            msg = 'Token File does not exist'
            raise IOError(msg)

    def is_stored(self):
        if not os.path.exists(self.filename):
            return False
        return True

class AuthOpenStack(object):
    def __init__(self, user_data, verbose = False):
        self.cmd = pycurl.Curl()
        self.user_data = user_data
        self.verbose = verbose
        self.result = {}

    def __set_cmdopt(self, endpoint):
        '''Sets command options'''
        self.cmd.setopt(pycurl.VERBOSE, self.verbose)
        self.cmd.setopt(pycurl.HTTPHEADER,
            ['Content-type: application/json'])
        self.cmd.setopt(pycurl.URL,
            'https://%s:%s/v2.0/tokens'
            % (self.user_data[endpoint]['auth_host'],
            self.user_data[endpoint]['auth_port']))
        self.cmd.setopt(pycurl.CAINFO,
            self.user_data['user']['user_proxy'])
        self.cmd.setopt(pycurl.CAPATH, 
            self.user_data['user']['ca_path'])
        self.cmd.setopt(pycurl.SSLCERT, 
            self.user_data['user']['user_proxy'])
        self.cmd.setopt(pycurl.SSLKEY, 
            self.user_data['user']['user_proxy'])
        self.cmd.setopt(pycurl.POSTFIELDS,
            '{"auth":{"voms": true}}')

    def perform_a_transfer(self, endpoint):
        '''Performs a transfer'''
        self.__set_cmdopt(endpoint)

        buf = cStringIO.StringIO()

        self.cmd.setopt(pycurl.WRITEFUNCTION, buf.write)
        try:
            self.cmd.perform()
            try:
                self.result = simplejson.loads(buf.getvalue())
            except ValueError, e:
                msg = str(e) + ' for ' + endpoint
                raise AuthOpenStackError(msg)
            buf.close()

        except pycurl.error, e:
            msg = str(e) + ' for ' + endpoint
            raise AuthOpenStackError(msg)

    def get_token(self):
        return self.result['access']['token']

    def get_id(self):
        return self.result['access']['token']['id']
    
    def get_service_catalog(self):
        return self.result['access']['serviceCatalog']

    def get_user(self):
        return self.result['acess']['user']

    def get_access(self):
        return self.result['access']

#{'access': {
## 'token': {'issued_at': '2013-04-03T17:20:36.497835', 'expires': '2013-04-04T17:20:36Z', 'id': '8a31b8a742b347df826a57c6168042c5', 'tenant': {'id': 'df37f5b1ebc94604964c2854b9c0551f', 'enabled': True, 'description': 'Instances of EGI Federated Clouds TF', 'name': 'EGI_FCTF'}},
## 'serviceCatalog': [{'endpoints_links': [], 'endpoints': [{'adminURL': 'https://egi-cloud.zam.kfa-juelich.de:8774/v2/df37f5b1ebc94604964c2854b9c0551f', 'region': 'FSDCloud', 'publicURL': 'https://egi-cloud.zam.kfa-juelich.de:8774/v2/df37f5b1ebc94604964c2854b9c0551f', 'internalURL': 'https://egi-cloud.zam.kfa-juelich.de:8774/v2/df37f5b1ebc94604964c2854b9c0551f', 'id': 'c1a76c9510dc402ca5d467f01009afb6'}], 'type': 'compute', 'name': 'nova'},
##                        {'endpoints_links': [], 'endpoints': [{'adminURL': 'https://egi-cloud.zam.kfa-juelich.de:9292/v1', 'region': 'FSDCloud', 'publicURL': 'https://egi-cloud.zam.kfa-juelich.de:9292/v1', 'internalURL': 'https://egi-cloud.zam.kfa-juelich.de:9292/v1', 'id': '2946429173954ebf9d4ba1faf970affc'}], 'type': 'image', 'name': 'glance'},
##                        {'endpoints_links': [], 'endpoints': [{'adminURL': 'https://egi-cloud.zam.kfa-juelich.de:8776/v1/df37f5b1ebc94604964c2854b9c0551f', 'region': 'FSDCloud', 'publicURL': 'https://egi-cloud.zam.kfa-juelich.de:8776/v1/df37f5b1ebc94604964c2854b9c0551f', 'internalURL': 'https://egi-cloud.zam.kfa-juelich.de:8776/v1/df37f5b1ebc94604964c2854b9c0551f', 'id': '6eab574309bf40059596574261ce1a40'}], 'type': 'volume', 'name': 'cinder'},
##                        {'endpoints_links': [], 'endpoints': [{'adminURL': 'https://egi-cloud.zam.kfa-juelich.de:8773/services/Admin', 'region': 'FSDCloud', 'publicURL': 'https://egi-cloud.zam.kfa-juelich.de:8773/services/Cloud', 'internalURL': 'https://egi-cloud.zam.kfa-juelich.de:8773/services/Cloud', 'id': 'db6b83b067c141e3bd0150d6dda4a888'}], 'type': 'ec2', 'name': 'ec2'},
##                        {'endpoints_links': [], 'endpoints': [{'adminURL': 'https://swift.zam.kfa-juelich.de:8888/v1', 'region': 'FSDCloud', 'publicURL': 'https://swift.zam.kfa-juelich.de:8888/v1/AUTH_df37f5b1ebc94604964c2854b9c0551f', 'internalURL': 'https://swift.zam.kfa-juelich.de:8888/v1/AUTH_df37f5b1ebc94604964c2854b9c0551f', 'id': 'b839833bd44f4c888da4537a20b95c54'}], 'type': 'object-store', 'name': 'swift'}, 
##                        {'endpoints_links': [], 'endpoints': [{'adminURL': 'https://egi-cloud.zam.kfa-juelich.de:35357/v2.0', 'region': 'FSDCloud', 'publicURL': 'https://egi-cloud.zam.kfa-juelich.de:5000/v2.0', 'internalURL': 'https://egi-cloud.zam.kfa-juelich.de:5000/v2.0', 'id': '43cae207e8794608a18cb3d5acbb2255'}], 'type': 'identity', 'name': 'keystone'}],
## 'user': {'username': 'IT/O=INFN/OU=Personal Certificate/L=CNAF/CN=Elisabetta Ronchieri', 'roles_links': [], 'id': 'IT/O=INFN/OU=Personal Certificate/L=CNAF/CN=Elisabetta Ronchieri', 'roles': [], 'name': 'IT/O=INFN/OU=Personal Certificate/L=CNAF/CN=Elisabetta Ronchieri'}}
#}


