#------------------------------------------------------------------------------
# Copyright 2008-2012 Istituto Nazionale di Fisica Nucleare (INFN)
# 
# Licensed under the EUPL, Version 1.1 only (the "Licence").
# You may not use this work except in compliance with the Licence. 
# You may obtain a copy of the Licence at:
# 
# http://joinup.ec.europa.eu/system/files/EN/EUPL%20v.1.1%20-%20Licence.pdf
# 
# Unless required by applicable law or agreed to in 
# writing, software distributed under the Licence is 
# distributed on an "AS IS" basis, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. 
# See the Licence for the specific language governing 
# permissions and limitations under the Licence.
#------------------------------------------------------------------------------

import os
import stat
import ConfigParser
import exceptions

class WrongConfigurationFile(exceptions.Exception):
    pass

class WrongConfigurationSettings(exceptions.Exception):
    pass

class WrongUserParameters(exceptions.Exception):
    pass

class GeneralError(exceptions.Exception):
    pass

class LoadConfig(object):
    def __init__(self, config_file=''):
        self.config_file = config_file
        self.parser = ConfigParser.SafeConfigParser()
        self.parameters = {}
        self.data = {}

    def __are_options_present(self, section):
        '''Verify the correctness of the options'''
        for option in self.parameters[section].keys():
            #print '%s: %s' % (section, option)
            if not self.parser.has_option(section, option):
                if option == 'user_cert':
                    if not self.parser.has_option(section, 'user_proxy'):
                        return False
                    else:
                        return True
                elif option == 'user_key':
                    if not self.parser.has_option(section, 'user_proxy'):
                        return False
                    else:
                        return True
                elif option == 'user_proxy':
                    if self.parser.has_option(section, 'user_cert') and self.parser.has_option(section, 'user_key'):
                        return True
                    else:
                        return False
                else:
                    return False

        return True

    def __are_sections_present(self):
        '''Verify the correctness of the sections'''
        for section in self.parameters.keys():
            #print 'section %s' % section
            if self.parser.has_section(section):
                if not self.__are_options_present(section):
                    #print 'no option'
                    return False
            else:
                #print 'no section'
                return False
        #print 'ok'
        return True

    def get_parser(self):
        return self.parser

    def get_data(self):
        '''get data'''
        try:
            self.parser.read(self.config_file)
            if not self.__are_sections_present():
                raise WrongConfigurationFile('Wrong Configuration file %s. '
                    % self.config_file +
                    'Please verify the file.')

            for section in self.parser.sections():
                self.data[section] = {}
                for option in self.parser.options(section):
                    self.data[section][option] = self.parser.get(section,
                        option)

        except ConfigParser.ParsingError, err:
            print 'Could not parse: ', err
        except ConfigParser.NoOptionError, err:
            print 'No such option: ', str(err)
        except ConfigParser.NoSectionError, err:
            print 'No such section: ', str(err)

        return self.data

class LoadUserConfig(LoadConfig):
    def __init__(self, config_file=''):
        super(LoadUserConfig, self).__init__(config_file=config_file)
        up = UserParameters()
        self.parameters = up.get_parameters()
        self.providers = up.get_providers()

    def __is_valid(self, option, value='test'):
        '''Verify the correctness of user configuration file settings'''
        if option == 'user_key':
            if not (os.path.isfile(value)
                and stat.S_IMODE(os.stat(value).st_mode) == 0400):
                return False
        elif option == 'user_cert':
            if not (os.path.isfile(value)
                and stat.S_IMODE(os.stat(value).st_mode) == 0600):
                return False
        elif option == 'user_proxy':
            if not (os.path.isfile(value)
                and stat.S_IMODE(os.stat(value).st_mode) == 0600):
                return False
        elif option == 'pub_key':
            if not (os.path.isfile(value)
                and stat.S_IMODE(os.stat(value).st_mode) == 0644):
                return False
        elif option == 'type':
            if value not in ('text/plain', 'text/occi'):
                return False
        #elif option == 'category':
        #    if value not in ('compute','storage','network'):
        #        return False
        return True

    def __check_mandatory_data(self, data, key):
        '''Check mandatory data'''

        for option in self.parameters[key].keys():
            if not self.__is_valid(option, data[option]):
                raise WrongConfigurationSettings(\
                    'Wrong Configuration Settings for option %s' % option)

    def __check_mandatory_providers_data(self, data):
        '''Check mandatory providers data'''
        for option in self.parameters['providers'].keys():
            if option in data['providers']:
                remove_space = data['providers'][option].replace(" ", "")
                tmp = remove_space.split(',')
                for endpoint in tmp:
                    try:
                        data[endpoint]
                    except KeyError:
                        msg = ('Wrong Configuration Settings. Section %s '
                            % endpoint +
                            ' is missing.')
                        raise WrongConfigurationSettings(msg)
                    else:
                        for p_option in self.providers[option]:
                            if p_option in data[endpoint].keys():
                                continue
                            else:
                                msg = ('Wrong Configuration Settings. '+
                                    'Option %s in Section %s is missing.'
                                    % (p_option, endpoint))
                                raise WrongConfigurationSettings(msg)
        
    def __check_mandatory_user_data(self, data):
        '''Check mandatory user data'''

        for option in self.parameters['user'].keys():
            if option == 'user_key':
                if 'user_proxy' in data.keys():
                    continue
            elif option == 'user_cert':
                if 'user_proxy' in data.keys():
                    continue
            elif option == 'user_proxy':
                if 'user_cert' in data.keys() and 'user_key' in data.keys():
                    continue
            if not self.__is_valid(option, data[option]):
                raise WrongConfigurationSettings(\
                    'Wrong Configuration Settings for option %s' % option)

    def get_data(self):
        '''Get conf data'''

        load = LoadConfig.get_data(self)
        for key, value in self.parameters.iteritems():
            try:
                load[key]
            except KeyError:
                msg = ('Wrong Configuration Settings. Section %s '
                    % key +
                    ' is missing.')
                raise WrongConfigurationSettings(msg)
            else:
                if key == 'user':
                    self.__check_mandatory_user_data(load[key])
                elif key == 'providers':
                    self.__check_mandatory_providers_data(load)
                else:
                    self.__check_mandatory_data(load[key], key)

        return load

class UserParameters(object):
    def __init__(self):
        self.parameters = {
            'marketplace': {'uri':'',
                'ns_dcterms':''},
            'providers': {
                'wnodes':'',
                'openstack':'',
                'opennebula':''},
            'user': {'vo': '',
                'user_key': '',
                'user_cert': '',
                'user_proxy': '',
                'ca_path': '/etc/grid-security/certificates'},
            'content': {'type': ''}
            }

        self.providers = {'wnodes': ['port', 'ns_port', 'ns_host','protocol'],
            'openstack': ['port', 'auth_port', 'auth_host', 'auth_path','protocol'],
            'opennebula': ['port', 'protocol']}

        self.cmd_options = ['--capath',
            '--cert',
            '--key']

    def get_parameters(self):
        '''Return user configuration parameters'''

        return self.parameters

    def get_providers(self, provider=''):
        '''Return user configuration providers'''

        if provider == '':
            return self.providers
        elif provider in self.providers.keys():
            return self.providers[provider]
        else:
            msg = '%s provider not supported' % provider
            raise WrongUserParameters(msg)

    def get_map(self):
        '''Return a map between user configuration options and command'''

        map_value = {}

        for option in self.cmd_options:
            if option == '--key':
                map_value['user_key'] = option
                map_value['user_proxy'] = option
            elif option == '--cert':
                map_value['user_cert'] = option
                map_value['user_proxy'] = option
            elif option == '--capath':
                map_value['ca_path'] = option
            

        return map_value
